import click
from indico_install.utils import options_wrapper
from indico_install.config import ConfigsHolder
from indico_install.helm import render, apply
from indico_install.setup import get_tls
from indico_install.kube.svc.restart import restart


@click.command("update")
@click.pass_context
@click.argument("service", required=True, type=click.Choice(["ssl"]))
@click.option(
    "-r",
    "--remote-configs",
    help="Remote GCS folder with configs to render from. Ex: 'latest' or 'master.24235243'",
)
@options_wrapper(check_input=True)
def update(ctx, service, *, input_yaml, remote_configs, **kwargs):
    """
    Function to update an existing service.

    Args:

      <SERVICE>: The service to update. Choices are:

        - ssl : update certificates
    """
    if service == "ssl":
        # Handle these here to avoid recursive imports
        config = ConfigsHolder(config=input_yaml)
        get_tls(config, redo=True)
        ctx.invoke(render, input_yaml=input_yaml, remote_configs=remote_configs, **kwargs)
        ctx.invoke(
            apply, service="app-edge-nginx-conf", input_yaml=input_yaml, **kwargs
        )
        ctx.invoke(restart, service="app-edge")



import click
from ipaddress import ip_network

from indico_install.infra.eks.config import session
from indico_install.infra.eks.infra_utils import gw_info, cluster_cidrs, is_covered_by


@session
def validate(info, session=None):
    gw_conf = gw_info(info, session=session)
    _val_indicoapifs(gw_conf["indico_fs"], session=session)
    _val_clientdatafs(gw_conf["client_fs"], session=session)
    _val_gateway(gw_conf["gw_arn"], info, session=session)



@session
def _val_clientdatafs(fs, session=None):
    # Make sure the fileshare for the client data is correctly configured
    assert fs["ObjectACL"] == "bucket-owner-full-control"
    assert not fs["ReadOnly"]
    role_arn = fs["Role"]
    try:
        _val_role(role_arn, session=session)
    except Exception as e:
        raise Exception(
            f"Invalid or incorrect IAM role policy found for Client S3 fileshare: {e}"
        )


@session
def _val_indicoapifs(fs, session=None):
    # Make sure the fileshare for the Indico data is correctly configured
    assert fs["ObjectACL"] == "bucket-owner-full-control"
    assert fs["ReadOnly"]

    # Check that there is an attached role to allow the fileshare access to the bucket
    role_arn = fs["Role"]
    try:
        _val_role(role_arn, session=session)
    except Exception as e:
        raise Exception(
            f"Invalid or incorrect IAM role policy found for Indico S3 fileshare {e}"
        )


@session
def _val_gateway(gateway_arn, conf, session=None):
    gw = session.SGWClient.describe_gateway_information(GatewayARN=gateway_arn)
    assert gw["GatewayState"] == "RUNNING"
    assert gw["GatewayType"] == "FILE_S3"
    gw_cache = session.SGWClient.describe_cache(GatewayARN=gateway_arn)
    assert (
        gw_cache["CacheAllocatedInBytes"] >= 100_000_000_000
    ), "Gateway cache must be >= 100GB"
    gw_ip = gw["GatewayNetworkInterfaces"][0]["Ipv4Address"]
    ec2 = session.EC2Client.describe_instances(
        Filters=[
            {
                "Name": "network-interface.addresses.private-ip-address",
                "Values": [gw_ip],
            }
        ]
    )["Reservations"][0]["Instances"][0]
    assert "xlarge" in ec2["InstanceType"]
    ec2_sgs = [sg["GroupId"] for sg in ec2["SecurityGroups"]]
    _val_sgs(ec2_sgs, conf, session=session)
    return gw


@session
def _val_sgs(sg_ids, conf, session=None):
    """
    Make sure the Gateway is exposed correctly to our cluster
    """
    sgs = session.EC2Client.describe_security_groups(GroupIds=sg_ids)["SecurityGroups"]
    all_ingress_rules = [
        {
            "from": perm.get("FromPort", 0),
            "to": perm.get("ToPort", 65535),
            "ips": [ip_network(ipr["CidrIp"]) for ipr in perm["IpRanges"]],
        }
        for sg in sgs
        for perm in sg["IpPermissions"]
    ]

    if not any(
        [
            r["from"] <= 2049 <= r["to"]
            and all(
                [
                    any([is_covered_by(s, ip) for ip in r["ips"]])
                    for s in cluster_cidrs(conf, session=session)
                ]
            )
            for r in all_ingress_rules
        ]
    ):
        click.secho("Gateway security rules may not expose NFS port 2049 to all cluster subnets. Please review (may be a false positive)", fg="yellow")


@session
def _val_role(role_arn, session=None):
    role_name = role_arn.split(":")[-1].split("/")[-1]
    role_policy = session.IAMClient.get_role(RoleName=role_name)["Role"]
    assert any(
        [
            s["Action"] == "sts:AssumeRole"
            and s["Principal"]["Service"] == "storagegateway.amazonaws.com"
            for s in role_policy["AssumeRolePolicyDocument"]["Statement"]
        ]
    ), f"Fileshare role {role_name} must allow AssumeRole to access S3"



import json
from click import secho
from indico_install.utils import run_cmd
from indico_install.infra.input_utils import download_indicoapi_data

FILE_SHARE_SECRET_NAME = "azure-storage-key"


def create(deployment_root, conf, upload=False):
    if upload:
        upload_indicoapi_data(deployment_root, conf)
    storage_acct_name = conf["cluster"]["storage_acct"]
    storage_acct_key = conf["cluster"]["storage_acct_key"]
    run_cmd(
        f"kubectl create secret generic {FILE_SHARE_SECRET_NAME} "
        f"--from-literal=azurestorageaccountname={storage_acct_name} "
        f"--from-literal=azurestorageaccountkey={storage_acct_key}"
    )


def upload_indicoapi_data(deployment_root, conf):
    data_path = download_indicoapi_data(deployment_root)
    cluster_info = conf["cluster"]
    api_data_fs = conf["clusterVolumes"]["rox"]["azureFile"]["shareName"]

    secho(f"Uploading data to Azure file share {api_data_fs}", fg="yellow")
    run_cmd(
        f"az storage file upload-batch --account-name {cluster_info['storage_acct']} "
        f"--account-key {cluster_info['storage_acct_key']} "
        f"-d {api_data_fs} -s {data_path} --max-connections 10"
    )
    secho("Upload complete!", fg="green")


def check(conf):
    cluster_info = conf["cluster"]
    api_data_fs = conf["clusterVolumes"]["rox"]["azureFile"]["shareName"]
    out = run_cmd(
        f"az storage file list --account-name {cluster_info['storage_acct']} "
        f"--account-key {cluster_info['storage_acct_key']} -s {api_data_fs}"
    )
    api_files = [f["name"] for f in json.loads(out)]
    assert (
        len(api_files) > 6 and "finetune_models" in api_files
    ), "Files have not been correctly uploaded to the indicoapi file share"



#!/usr/bin/env python3
from pathlib import Path
from click import secho

import click
import yaml

from indico_install.infra.init import init
from indico_install.infra.eks import cluster, config, database, network, storage
from indico_install.config import ConfigsHolder, merge_dicts
from indico_install.utils import run_cmd, options_wrapper

SOURCES = [network, storage, database, cluster]


@click.group("eks")
@click.pass_context
def eks(ctx):
    """
    Indico infrastructure setup and validation for AWS Kubernetes Service
    """
    ctx.ensure_object(dict)
    if not ((config.access_key and config.access_secret) or config.aws_profile):
        secho("Missing AWS credentials for EKS", fg="yellow")
    else:
        secho(
            f"Using AWS profile: {config.aws_profile}"
            if config.aws_profile
            else f"Using AWS access key: {config.access_key[:5]}****",
            fg="blue",
        )


eks.command("init")(init(__name__))


@eks.command("check")
@click.pass_context
@options_wrapper(check_input=True)
def check(ctx, *, input_yaml, **kwargs):
    """Validate all resources"""
    try:
        ctx.obj["SESSION"] = config.Session()
    except Exception as e:
        secho(f"AWS Session could not be generated: {e}", fg="red")
        return
    if not input_yaml.is_file():
        click.secho(f"Could not find {input_yaml}.", fg="red")
        return
    failed = 0
    user_conf = ConfigsHolder(config=input_yaml)
    config.ask_for_infra_input(user_conf)
    user_conf.save()
    for resource in SOURCES:
        resource_name = resource.__name__
        secho(f"Validating {resource_name}...")
        try:
            resource.validate(user_conf, session=ctx.obj["SESSION"])
        except Exception as e:
            secho(f"{resource_name} NOT OK! Error: {e}\n", fg="red")
            failed += 1
        else:
            secho(f"{resource_name} OK!\n", fg="green")
    secho(
        f"Validation complete: {failed} errors",
        fg="red" if failed else "green",
        bold=True,
    )


@eks.command("create")
@click.pass_context
@click.option("--eksctl-file", help="eksctl config file used to create cluster")
@options_wrapper(check_input=True)
def create(ctx, *, input_yaml, eksctl_file, **kwargs):
    """
    Install the nvidia driver on the cluster
    """
    run_cmd(
        "kubectl create -f https://raw.githubusercontent.com/NVIDIA/k8s-device-plugin/1.0.0-beta/nvidia-device-plugin.yml"
    )
    user_conf = ConfigsHolder(input_yaml)

    if not eksctl_file or not Path(eksctl_file).is_file():
        config.ask_for_infra_input(user_conf)
    else:
        with open(eksctl_file, "r") as conf_file:
            conf = yaml.safe_load(conf_file)

        cluster_name = conf["metadata"]["name"]
        nodegroup = conf["nodeGroups"][0]
        user_conf["cluster"] = user_conf.get("cluster", {}) or {}
        user_conf["cluster"]["name"] = cluster_name

        if nodegroup.get("privateNetworking", False):
            user_conf["services"] = merge_dicts(
                user_conf.get("services", {}),
                {
                    "app-edge": {
                        "values": {
                            "annotations": {
                                "service.beta.kubernetes.io/aws-load-balancer-internal": "0.0.0.0/0"
                            }
                        }
                    }
                },
            )
    user_conf.save()



from pathlib import Path
import click
import pkg_resources

from indico_install.config import CLUSTER_NAME


def init(location):
    def wrapper():
        values_dir = Path("values")
        if not values_dir.exists():
            values_dir.mkdir(exist_ok=True, parents=True)

        dest_file = values_dir / f"{CLUSTER_NAME}.yaml"
        if dest_file.exists():
            click.secho(
                f"{dest_file.resolve()} already exists. "
                "Please back this file up before running init.",
                fg="red",
            )
            return

        template = pkg_resources.resource_string(location, "cluster.yaml")
        with open(dest_file, "wb") as f:
            f.write(template)

        click.secho(f"Created {dest_file}", fg="green")

    return wrapper



import re

import click
from click import confirm, secho
import tempfile
from pathlib import Path
from indico_install.config import ConfigsHolder
from indico_install.helm.apply import apply
from indico_install.helm.render import render, resolve_all
from indico_install.kube.svc.restart import restart
from indico_install.utils import run_cmd, options_wrapper

from indico_install.sync.utils import APP_MAP, get_app_hashes, get_non_matching_images

REVERSE_APP_MAP = {value: key for key, value in APP_MAP.items()}


@click.group("push")
@click.pass_context
def push(ctx):
    """Push hash(es) and image(s) to the cluster"""
    ctx.ensure_object(dict)


@push.command("all")
@click.pass_context
@options_wrapper(check_services=True, check_input=True)
def push_all(
    ctx, *, input_yaml, services_yaml, cluster, yes, deployment_root, forwarded=True
):
    """
    All service and frontend hashes in the services yaml
    will be pushed to the current cluster.
    """
    if forwarded is True:
        ctx.invoke(push_all, forwarded=False)

    configs = ConfigsHolder(config=services_yaml)
    for (
        _,
        resource,
        deployment,
        saved_image,
        cluster_image,
    ) in get_non_matching_images(configs):
        secho(f"{deployment}:\nOn Cluster: {cluster_image}\nOn Disk: {saved_image}")

        if yes or confirm("Push new image?"):
            secho(
                run_cmd(
                    f"kubectl set image --record=true {resource}/{deployment} "
                    f"{deployment}=gcr.io/new-indico/{saved_image}"
                ),
                fg="green",
            )
        else:
            secho(f"Skipping {deployment}", fg="yellow")

    # Frontend
    ctx.invoke(
        render,
        service="app-edge-nginx-conf",
        deployment_root=deployment_root,
        cluster=cluster,
        input_yaml=input_yaml,
        services_yaml=services_yaml,
    )
    ctx.invoke(
        apply, service="app-edge-nginx-conf", deployment_root=deployment_root, yes=yes
    )
    ctx.invoke(restart, service="app-edge")
    secho("Done pushing changes", fg="green")


@push.command("svc")
@click.pass_context
@click.argument("svc")
@click.option("--exclude", default=None)
@options_wrapper(check_services=True)
def push_svc(ctx, svc, *, services_yaml, yes, image=None, exclude=None, **kwargs):
    resources = []
    for resource_type in ("deployment", "statefulset"):
        options = run_cmd(
            f"kubectl get {resource_type} -o wide "
            f"| grep indico "
            f"| awk '{{print $1}}'| grep {svc}",
            silent=True,
        )

        resources.extend(
            [(resource_type, resource) for resource in options.split("\n") if resource]
        )

    if exclude is not None:
        resources = [
            (resource_type, resource)
            for resource_type, resource in resources
            if not re.match(rf".*{exclude}.*", resource)
        ]

    click.secho(f"Matched {[x[1] for x in resources]}", fg="green")
    configs = ConfigsHolder(config=services_yaml)
    resolve_all(configs)
    resources = [
        (r_type, resource, configs["services"][resource]["values"]["image"])
        for r_type, resource in resources
    ]

    for resource_type, resource, image in resources:
        secho(f"{resource_type}/{resource}:\n Update with: {image}")

        if yes or confirm("Push new image?"):
            secho(
                run_cmd(
                    "kubectl set image --record=true "
                    f"{resource_type}/{resource} {resource}=gcr.io/new-indico/{image}",
                    silent=True,
                )
            )
        else:
            secho(f"Skipping {resource}", fg="yellow")


@push.command("image")
@click.pass_context
@click.argument("image_name")
@click.argument("image", required=False, default=None)
@options_wrapper()
def push_image(ctx, image_name, *, services_yaml, yes, image=None, **kwargs):
    """
    Update <IMAGE_NAME> matching deployment/statefulset
    with image saved in current cluster config.
    Ex. indico push image noct
    """
    resources = []
    for resource_type, column in [("deployment", 7), ("statefulset", 5)]:
        options = run_cmd(
            f"kubectl get {resource_type} -o wide "
            f"| grep indico "
            f"| awk '${column} ~ /{image_name}/ {{print $1}}'",
            silent=False,
        )

        resources.extend(
            [(resource_type, resource) for resource in options.split("\n") if resource]
        )

    if image is None:
        if not Path(services_yaml).is_file():
            secho(f"Could not find {services_yaml}", fg="red")
            return
        configs = ConfigsHolder(config=services_yaml)
        image = configs["images"][image_name.replace("-", "").replace("_", "")]

    for resource_type, resource in resources:
        secho(f"{resource_type}/{resource}:\n Update with: {image}")

        if yes or confirm("Push new image?"):
            secho(
                run_cmd(
                    "kubectl set image --record=true "
                    f"{resource_type}/{resource} {resource}=gcr.io/new-indico/{image}",
                    silent=True,
                )
            )
        else:
            secho(f"Skipping {resource}", fg="yellow")


@push.command("client")
@click.pass_context
@click.argument("hash")
def push_client(ctx, hash):
    """Update frontend <HASH>. Will also update app_edge"""
    output = run_cmd(
        """kubectl get configmap app-edge-nginx-conf -o json | jq '.data["nginx.conf"]'""",
        silent=True,
    )

    nginx_conf = re.sub(
        "set \$clientversion [^;]*;",
        f"set $clientversion {hash};",
        output,
        flags=re.MULTILINE,
    )

    with tempfile.NamedTemporaryFile() as temp:
        temp.write(str.encode('{"data": {"nginx.conf":' + f"{nginx_conf}" + "}}')"))
        command = (
            f'kubectl patch configmap app-edge-nginx-conf --patch "$(cat {temp.name})"'
        )
        secho(run_cmd(command, silent=True))
        ctx.invoke(restart, service="app-edge")
        secho("Done pushing changes", fg="green")



from pathlib import Path
import copy
import os
import re
import yaml
import tempfile
from contextlib import contextmanager
from collections import defaultdict

import click

from indico_install.config import (
    ConfigsLoader,
    merge_dicts,
    REMOTE_TEMPLATES_PATH,
    yaml,
)
from indico_install.setup import setup
from indico_install.utils import run_cmd, options_wrapper

var_regex = re.compile(r"<\!var:([^<>]*)>")
lookup_regex = re.compile(r"<\!lookup:([^<>]*)>")


@contextmanager
def _resolve_values(values):
    f = tempfile.NamedTemporaryFile(mode="w")
    yaml.dump(dict(values.items()), f, default_flow_style=False)
    yield f" -f {f.name}"
    f.close()


def _resolve_template(template_values, variables, conf):
    if isinstance(template_values, dict):
        new_values = {}
        for key, value in template_values.items():
            new_values[_resolve_template(key, variables, conf)] = _resolve_template(
                value, variables, conf
            )
        return new_values
    elif isinstance(template_values, list):
        return [_resolve_template(value, variables, conf) for value in template_values]
    elif isinstance(template_values, str):
        var_matches = var_regex.findall(template_values)
        lookup_matches = lookup_regex.findall(template_values)
        while var_matches or lookup_matches:
            for match in var_matches:
                parts = match.split("=", 1)
                if len(parts) > 1:
                    default = str(parts[1])
                else:
                    default = ""
                template_values = template_values.replace(
                    f"<!var:{match}>", str(variables.get(parts[0], default))
                )
            for match in lookup_matches:
                look_up_keys = match.split("|")
                target_value = dict(conf)
                parts = look_up_keys[-1].split("=", 1)
                if len(parts) > 1:
                    default = str(parts[1])
                    look_up_keys[-1] = parts[0]
                else:
                    default = ""
                for look_up_key in look_up_keys:
                    target_value = target_value.get(look_up_key, {})
                template_values = template_values.replace(
                    f"<!lookup:{match}>",
                    str(target_value) if target_value != {} else default,
                )
            var_matches = var_regex.findall(template_values)
            lookup_matches = lookup_regex.findall(template_values)

    return template_values


def helm_render(
    deployment_root,
    templates_dir,
    group,
    template,
    values,
    values_files="",
    enabled_resource_validation=True,
):
    generated_dir = deployment_root / "generated"
    click.echo(f"Generating {values['name']}.yaml")
    with _resolve_values(values) as resolved_str:
        results = run_cmd(
            f"helm template {values_files} {templates_dir / group} --execute templates/{template}.yaml{resolved_str}",
            silent=True,
        )
        if enabled_resource_validation:
            validate_resource_constraints(results)

    with open(generated_dir / f"{values['name']}.yaml", "w") as f:
        f.write(results)


def validate_resource_constraints(generated_results):
    """
    validate that a generated template has the required resource section
    """
    results = yaml.safe_load_all(generated_results)
    deploys = [
        result
        for result in results
        if isinstance(result, dict)
        and result.get("kind") in ("Deployment", "StatefulSet", "DaemonSet")
    ]
    for deploy in deploys:
        for container in deploy["spec"]["template"]["spec"]["containers"]:
            if "resources" not in container:
                raise Exception(
                    f"No resource specifications set for {deploy['kind']}: {deploy['metadata']['name']}"
                )


def resolve_templates(templates):
    unresolved_templates = lambda: {
        template_name: template
        for template_name, template in templates.items()
        if "<!template>" in template
    }
    templates_to_resolve = unresolved_templates()
    while templates_to_resolve:
        for template_name, template in templates_to_resolve.items():
            template_to_merge = template.pop("<!template>")
            templates[template_name] = merge_dicts(
                copy.deepcopy(templates[template_to_merge]), templates[template_name]
            )
        templates_to_resolve = unresolved_templates()


def resolve_all(configs):
    resolve_templates(configs["_templates"])

    for resource, helm_info in configs["services"].items():
        if "<!template>" in helm_info:
            template_name = helm_info.pop("<!template>")
            configs["services"][resource] = merge_dicts(
                copy.deepcopy(configs["_templates"][template_name]), helm_info
            )
            # Attach additional values
        configs["services"][resource]["values"] = configs["services"][resource].get(
            "values", {}
        )
        configs["services"][resource] = _resolve_template(
            configs["services"][resource], configs["services"][resource], configs
        )

    configs.update(
        _resolve_template(
            {k: v for k, v in configs.items() if k not in ("_templates", "services")},
            {},
            configs,
        )
    )


def format_resources(helm_info):
    """
    If resource contraints are specified within a 'resources' key, set that
    value in 'limits' and 'requests'. Else, use the value set in the limits
    key; otherwise, rely on the values.yaml default for a value
    """
    if (
        "requests" in helm_info["values"]["resources"]
        or "limits" in helm_info["values"]["resources"]
    ):
        click.secho(
            "Usage of 'limits' and 'requests' in templates is deprecated; please specify resource constraints in a single 'resources' block",
            fg="yellow",
        )
    updated_resources = defaultdict(dict, helm_info["values"]["resources"])
    for resource in ("cpu", "memory", "nvidia.com/gpu"):
        if helm_info["values"]["resources"].get(resource):
            val = updated_resources.pop(resource)
            updated_resources["limits"].update({resource: val})
            updated_resources["requests"].update({resource: val})

        elif resource in updated_resources.get("limits"):
            updated_resources["requests"][resource] = updated_resources["limits"][
                resource
            ]
        elif resource in updated_resources.get("requests"):
            updated_resources["limits"][resource] = updated_resources["requests"][
                resource
            ]

    helm_info["values"]["resources"] = dict(updated_resources)
    return helm_info


def render_from_local(
    deployment_root, templates_dir, cluster, services_yaml, input_yaml, service=None
):
    generated = deployment_root / "generated"
    click.secho(f"clearing {generated}", fg="yellow")
    os.makedirs(generated, exist_ok=True)
    run_cmd(f"rm -rf {generated/ '*.yaml'}", silent=True)

    setup(input_yaml)
    # images will be "updated" by Indico and contains cluster defaults
    # input is customer overrides
    configs = ConfigsLoader(services_yaml, input_yaml)
    resolve_all(configs)

    with _resolve_values(configs) as resolved_str:
        for resource, helm_info in configs["services"].items():
            if service and service not in resource:
                continue

            helm_info["values"] = helm_info.get("values", {})
            helm_info["values"]["name"] = resource
            configs["services"][resource] = helm_info
            if helm_info.pop("<!disabled>", None):
                continue
            extra_keys = {
                k: v
                for k, v in helm_info.items()
                if k not in ("group", "template", "values")
            }
            helm_info["values"] = merge_dicts(extra_keys, helm_info["values"])
            [helm_info.pop(k) for k in extra_keys.keys()]

            if "resources" in helm_info["values"]:
                helm_info = format_resources(helm_info)

            try:
                helm_render(
                    deployment_root,
                    templates_dir,
                    values_files=f" {resolved_str}",
                    enabled_resource_validation=(
                        not configs.get("disableResourceLimits", False)
                    ),
                    **helm_info,
                )
            except Exception as e:
                click.secho(f"Unable to render template for {resource}: {e}")

    configs.save(deployment_root / "configuration" / f"{cluster}.yaml")


def _resolve_remote(deployment_root, remote_path):
    """
    Download remote templates and services yaml and unpack
    Unpack if necessary, and validate contents (error if validation fails)
    Return the location of the local, unpacked templates dir
    """
    remote_templates_path = REMOTE_TEMPLATES_PATH + remote_path + "/templates.tar.gz"
    remote_services_yaml = REMOTE_TEMPLATES_PATH + remote_path + "/services.yaml"
    local_directory = (
        deployment_root
        / "remote_configs"
        / "".join(c for c in str(remote_path) if c.isalnum)
    )
    if not local_directory.parent.exists():
        local_directory.parent.mkdir(exist_ok=True, parents=True)

    click.secho(f"Downloading remote configs to {local_directory}", fg="yellow")
    if local_directory.is_dir():
        run_cmd(f"rm -rf {local_directory}", silent=True)
    os.makedirs(local_directory / "templates", exist_ok=True)

    run_cmd(
        f"wget {remote_templates_path} -O - | tar -xz -C {local_directory / 'templates'}"
    )
    run_cmd(f"wget {remote_services_yaml} -O {local_directory / 'services.yaml'}")
    return (local_directory / "services.yaml", local_directory / "templates")


@click.command("render")
@click.pass_context
@click.argument("service", required=False)
@click.option(
    "-r",
    "--remote-configs",
    help="Remote GCS folder with configs to render from. Ex: 'latest' or 'master.24235243'",
)
@options_wrapper(check_input=True)
def render(
    ctx,
    service=None,
    *,
    deployment_root,
    cluster,
    input_yaml,
    services_yaml,
    remote_configs,
    **kwargs,
):
    """
    Render Helm templates from the "templates" directories.

    Only render the template for services with names containing <SERVICE> if provided
    """
    deployment_root = Path(deployment_root)
    templates_dir = deployment_root / "templates"
    if remote_configs:
        services_yaml, templates_dir = _resolve_remote(deployment_root, remote_configs)

    if not services_yaml.is_file():
        click.secho(f"Could not find {services_yaml}.", fg="red")
        return

    render_from_local(
        deployment_root,
        templates_dir,
        cluster,
        services_yaml,
        input_yaml,
        service=service,
    )
    click.secho("All set!", fg="green", bold=True)



import click
from indico_install.utils import run_cmd, options_wrapper
from indico_install.config import ConfigsHolder, merge_dicts

@click.command("delete")
@click.pass_context
@click.argument("service", required=True, type=str)
@click.option("--save/--no-save", default=True, show_default=True, help="Set disabled on the services in the cluster yaml")
@options_wrapper()
def delete(ctx, service, *, save, input_yaml, yes, **kwargs):
    """
    Delete a K8S deployment/statefulset/service

    ARGS:
        <SERVICE> grep string of deployments, statefulsets, services to delete
    """
    updated_svcs = []
    for svc_type in ["deployment", "statefulset", "service"]:
        out = run_cmd(
            """kubectl get %s --no-headers | grep "%s" | awk '{print $1}'"""
            % (svc_type, service),
            silent=True,
        )
        if not out:
            continue
        for _svc in out.splitlines():
            if yes or click.confirm(f"Ok to delete {svc_type} {_svc}"):
                click.secho(
                    run_cmd(f"kubectl delete {svc_type} {_svc}"),
                    fg="green",
                )
                updated_svcs.append(_svc)

    if save and updated_svcs:
        if not input_yaml.is_file():
            click.secho(
                f"Could not find {input_yaml}. Unable to save deletion", fg="yellow"
            )
            return
        conf = ConfigsHolder(config=input_yaml)
        updated_svcs_dict = {
            _svc: {"<!disabled>": True} for _svc in updated_svcs
        }
        conf["services"] = merge_dicts(conf.get("services", {}), updated_svcs_dict)
        conf.save()


@click.command("scale")
@click.pass_context
@click.argument("service", required=True, type=str)
@click.argument("amount", required=True, type=int)
@options_wrapper()
def scale(ctx, service, amount, *, input_yaml, yes, **kwargs):
    """
    Scale a K8S cluster deployment or statefulset

    ARGS:
        <SERVICE> grep string of deployments and statefulsets to scale

        <AMOUNT> number of pods to create
    """
    updated_svcs = []
    for svc_type in ["deployment", "statefulset"]:
        out = run_cmd(
            """kubectl get %s --no-headers | grep "%s" | awk '{print $1}'"""
            % (svc_type, service),
            silent=True,
        )
        if not out:
            continue
        for _svc in out.splitlines():
            click.secho(
                run_cmd(f"kubectl scale --replicas={amount} {svc_type} {_svc}"),
                fg="green",
            )
            updated_svcs.append(_svc)

    if updated_svcs and yes:
        if not input_yaml.is_file():
            click.secho(
                f"Could not find {input_yaml}. Unable to save new scale", fg="yellow"
            )
            return
        conf = ConfigsHolder(config=input_yaml)
        updated_svcs_dict = {
            _svc: {"values": {"replicas": amount}} for _svc in updated_svcs
        }
        conf["services"] = merge_dicts(conf.get("services", {}), updated_svcs_dict)
        conf.save()



import re
import tarfile
from pathlib import Path

import click
from click import secho
from indico_install.config import ConfigsHolder
from indico_install.infra.input_utils import auth_with_gsutil
from indico_install.utils import options_wrapper, run_cmd, string_to_tag


@click.group("package")
@click.pass_context
def package(ctx):
    """
    Package indico code versions spec'd in a local services.yaml
    """
    pass


@package.group("frontend")
@click.pass_context
def frontend(ctx):
    """
    Handle frontend package operations
    """
    pass


@frontend.command("load")
@click.pass_context
@click.option("--release", required=True, help="Release to download")
@options_wrapper(check_input=True)
def frontend_load(ctx, release=None, deployment_root=None, input_yaml=None, **kwargs):
    """
    Download frontend static files TAR and unzip to local directory
    """
    remote_fe_files_path = f"https://storage.googleapis.com/appstack-clients/client-onprem/frontend_package_{release}.tar.gz"

    local_frontend_files_dir = Path(deployment_root) / "frontend_package"
    local_frontend_files_dir.mkdir(exist_ok=True)
    run_cmd(f"wget {remote_fe_files_path} -O - | tar -xz -C {local_frontend_files_dir}")

    input_yaml = ConfigsHolder(input_yaml)
    input_yaml["frontend"]["inCluster"] = True
    input_yaml.save()


@frontend.command("save")
@click.option("--tag", required=True, help="Tag to attach to the TAR")
@click.pass_context
@options_wrapper(check_services=True)
def frontend_save(ctx, *, tag, deployment_root, services_yaml, **kwargs):
    """
    Packages together all static frontend files
    from hashes listed in services yaml
    """
    tag = string_to_tag(tag)
    services = ConfigsHolder(services_yaml)
    deployment_root = Path(deployment_root)
    frontend_hashes = services["frontend"]["hash"]

    frontend_files_dir = deployment_root / f"frontend_package_{tag}"
    for client, _hash in frontend_hashes.items():
        secho(f"Downloading FE {client} {_hash}...")
        client_dir = frontend_files_dir / client
        client_dir.mkdir(parents=True, exist_ok=True)
        run_cmd(
            f"gsutil -m cp -r gs://appstack-clients/client-onprem/{client}/{_hash} {client_dir}"
        )

    tar_file_path = deployment_root / f"frontend_package_{tag}.tar.gz"
    with tarfile.open(tar_file_path, "w:gz") as tar:
        tar.add(frontend_files_dir, arcname="")

    secho(f"FE files ready in {tar_file_path}", fg="green")
    run_cmd(
        f"gsutil cp {tar_file_path} gs://appstack-clients/client-onprem/{tar_file_path.name}"
    )
    return tar_file_path


@package.group("docker")
@click.pass_context
def docker(ctx):
    """
    Handle docker image package operations
    """
    pass


def create_ecr_repos(repos):
    curr_repositories = run_cmd(
        'aws ecr describe-repositories --no-paginate --output text --query "repositories[].repositoryName"'
    ).split()
    new_repos = set(repos) - set(curr_repositories)
    for repo in new_repos:
        run_cmd(f"aws ecr create-repository --repository-name {repo}")


REPO_CREATION_COMMANDS = {"ecr": create_ecr_repos}


@docker.command("load")
@click.pass_context
@click.option("--release", required=True, help="Release to download")
@click.option(
    "--download/--no-download",
    default=False,
    show_default=True,
    help="Download image TAR and list from remote",
)
@click.option(
    "--remote-repo", help="Push images to this repository URL using docker push"
)
@click.option(
    "--create-repos",
    type=click.Choice(list(REPO_CREATION_COMMANDS.keys())),
    help="Create missing repositories. Only invoked with --remote-repo",
)
@options_wrapper(check_input=True)
def docker_load(
    ctx,
    release=None,
    download=False,
    remote_repo=None,
    create_repos=None,
    yes=False,
    deployment_root=None,
    input_yaml=None,
    **kwargs,
):
    """
    Loads docker images from a TAR if specified into your local docker repo

    Will retag and push images listed in IMAGES if a remote-repo is specified
    """
    deployment_root = Path(deployment_root)
    auth_with_gsutil(deployment_root)
    images_tar = deployment_root / f"indico_images_{release}.tar.gz"
    images_file = deployment_root / f"indico_image_list_{release}.txt"
    if download:
        run_cmd(f"gsutil cp gs://indico-images/{images_tar.name} {deployment_root}")
        run_cmd(f"gsutil cp gs://indico-images/{images_file.name} {deployment_root}")

        assert (
            images_tar.is_file()
        ), f"Could not find TAR file {images_tar}. Is the release correct?"
        secho(f"Going to run command: `docker load < {images_tar}`", fg="yellow")
        if remote_repo or click.prompt(
            "Load docker TAR? This can take many minutes. You may also say No here and run the command in the background"
        ):
            run_cmd(f"docker load < {images_tar}")

    if remote_repo:
        assert images_file.is_file(), f"Could not find image list file {images_file}"

        remote_repo = remote_repo[:-1] if remote_repo.endswith("/") else remote_repo

        input_yaml = ConfigsHolder(input_yaml)
        input_yaml["dockerRegistry"] = remote_repo
        input_yaml.save()

        with open(images_file, "r") as imgf:
            images_list = [l.strip() for l in imgf.readlines()]

        new_tags = []
        for img in images_list:
            raw_img_name = img.split("/")[-1]
            new_tags.append(f"{remote_repo}/{raw_img_name}")

        repos = set([nt.split("/", 1)[1].split(":", 1)[0] for nt in new_tags])

        if create_repos:
            REPO_CREATION_COMMANDS[create_repos](repos)
        else:
            repos = "\n".join(repos)
            secho(
                f"Expecting the following repositories to exist. Use the --create-repos flag to create as necessary:\n{repos}\n"
            )

        for img, new_tag in zip(images_list, new_tags):
            if yes or click.prompt(f"Ready to tag and push {img} as {new_tag}? [yN]"):
                run_cmd(f"docker tag {img} {new_tag}")
                run_cmd(f"docker push {new_tag}")


@docker.command("save")
@click.pass_context
@click.option("--tag", required=True, help="Tag to attach to the TAR")
@options_wrapper(check_services=True)
def docker_save(ctx, *, tag, deployment_root, services_yaml, **kwargs):
    """
    Packages all docker images listed in services yaml
    and any specified in local templates

    Requires authed gsutil for upload
    """
    tag = string_to_tag(tag)
    deployment_root = Path(deployment_root)
    templates_dir = deployment_root / "templates"
    services = ConfigsHolder(services_yaml)
    docker_registry = "gcr.io/new-indico"

    images = set(
        [f"{docker_registry}/{image}" for image in services["images"].values()]
    )

    static_images = run_cmd(
        f"grep 'image: ' -r {templates_dir} -h | grep -v '.Values.image' | grep -v 'dockerRegistry }}}}'",
        silent=True,
    ).splitlines()
    static_images = [si.split("image: ", 1)[1] for si in static_images]
    for static_image in static_images:
        if " default " in static_image:
            static_image = re.sub(r'^.*default "(.*)" }}/', r"\g<1>/", static_image)
        images.add(static_image)

    image_list = " ".join(images)
    secho("Pulling:\n" + "\n".join(images))

    docker_tar_path = deployment_root / f"indico_images_{tag}.tar.gz"
    image_file_path = deployment_root / f"indico_image_list_{tag}.txt"
    for img in images:
        run_cmd(f"docker pull {img}")

    run_cmd(f"docker save {image_list} | gzip > {docker_tar_path}")
    with open(image_file_path, "w") as image_file:
        image_file.writelines(f"{img}\n" for img in images)
    secho(
        f"Docker images saved to {docker_tar_path}. Image list is {image_file_path}",
        fg="green",
    )
    run_cmd(f"gsutil cp {docker_tar_path} gs://indico-images/{docker_tar_path.name}")
    run_cmd(f"gsutil cp {image_file_path} gs://indico-images/{image_file_path.name}")
    return (image_file_path, docker_tar_path)



#!/usr/bin/env python3
import click


from indico_install.config import ConfigsHolder
from indico_install.infra.init import init
from indico_install.utils import options_wrapper
from .setup import aks_setup, ask_for_infra_input
from . import storage, cluster


@click.group("aks")
@click.pass_context
def aks(ctx):
    """
    Indico infrastructure setup and validation for Azure Kubernetes Service
    """
    ctx.ensure_object(dict)


aks.command("init")(init(__name__))


@aks.command("check")
@click.pass_context
@options_wrapper(check_input=True)
def check(ctx, *, deployment_root, input_yaml, **kwargs):
    """
    Check the state of an existing cluster to validate
    that it meets certain requirements
    """
    aks_setup(deployment_root)
    conf = ConfigsHolder(input_yaml)
    ask_for_infra_input(conf)
    cluster.check(conf)
    storage.check(conf)


@aks.command("create")
@click.pass_context
@click.option(
    "--upload/--no-upload",
    default=False,
    help="Upload Indico API data to the Indico file share",
    show_default=True,
)
@options_wrapper(check_input=True)
def create(ctx, upload=False, *, deployment_root, input_yaml, **kwargs):
    """
    Configure your AKS installation
    """
    aks_setup(deployment_root)
    conf = ConfigsHolder(input_yaml)
    ask_for_infra_input(conf)
    cluster.create(conf)
    storage.create(deployment_root, conf, upload=upload)



from pathlib import Path
import click
from click import prompt

from indico_install.config import ConfigsHolder
from indico_install.utils import run_cmd, options_wrapper
from indico_install.infra.init import init

SCRIPT = Path(__file__).parent / "bin" / "indico_infra.sh"


@click.group("sn")
@click.pass_context
def single_node(ctx):
    """
    Create and validate Indico platform installation on a single node
    """
    pass


single_node.command("init")(init(__name__))


@single_node.command("check")
@click.pass_context
@options_wrapper(check_input=True)
def check(ctx, *, input_yaml, **kwargs):
    """Validate local MicroK8S cluster installation"""
    conf = ConfigsHolder(config=input_yaml)
    run_cmd([str(SCRIPT.resolve()), "check"], tty=True)

    rw_dir = Path(conf["clusterVolumes"]["rwx"]["hostPath"]["path"])
    ro_dir = Path(conf["clusterVolumes"]["rox"]["hostPath"]["path"])
    assert (
        rw_dir and rw_dir.is_dir()
    ), "Directory selected for read-write data does not exist!"
    assert (
        ro_dir and ro_dir.is_dir()
    ), "Directory selected for read-only data does not exist!"


@single_node.command("create")
@click.pass_context
@options_wrapper(check_input=True)
def create(ctx, *, input_yaml, **kwargs):
    """Install local MicroK8S cluster"""
    conf = ConfigsHolder(config=input_yaml)
    run_cmd([str(SCRIPT.resolve()), "create"], tty=True)
    ask_for_infra_input(conf)

    rw_dir = Path(conf["clusterVolumes"]["rwx"]["hostPath"]["path"])
    ro_dir = Path(conf["clusterVolumes"]["rox"]["hostPath"]["path"])

    rw_dirs = " ".join(
        [
            str(rw_dir / sub_dir)
            for sub_dir in ("postgres_data", "rainbow", "finetune_models")
        ]
    )
    run_cmd(f"sudo mkdir -pm 777 {rw_dirs} {str(ro_dir)}".split(), tty=True)


def ask_for_infra_input(conf):
    click.secho(
        "Networking information.\n"
        "If connecting just from your local machine, use 127.0.0.1.\n"
        "For external access, use the computer hostname, and configure DNS to resolve this hostname.\n",
        fg="yellow"
    )
    conf["parentDomain"] = prompt(
        "Application domain",
        default=(conf["parentDomain"] or ".indico.io")
    )
    conf["publicIP"] = prompt(
            "The IP you want to use to connect. Example: 127.0.0.1",
            default=(conf["publicIP"] or "127.0.0.1")
    )
    app_port = prompt(
            "Application port - must be greater than 30000",
            default=(conf["services"]["app-edge"]["values"]["ports"]["https_port"] or 30001)
    )
    curr_ports = conf["services"]["app-edge"]["values"]["ports"]
    for i, port_name in enumerate(["https_port", "http_port", "http_api_port"]):
        curr_ports[port_name] = curr_ports[port_name] or app_port + i
    conf["services"]["app-edge"]["values"]["ports"] = curr_ports

    conf["publicDomain"] = f"{conf['publicIP']}:{app_port}"

    click.secho("Volume paths to save data on your machine", fg="yellow")
    conf["clusterVolumes"]["rwx"]["hostPath"]["path"] = prompt(
        "Directory for all indico data - should have at least 800GB available",
        default=conf["clusterVolumes"]["rwx"]["hostPath"].get("path", None),
    )

    conf["clusterVolumes"]["rox"]["hostPath"]["path"] = prompt(
        "Directory for read-only-v7 API data - should have 120GB available",
        default=conf["clusterVolumes"]["rox"]["hostPath"].get("path", None),
    )
    conf.save()



import os
from collections import OrderedDict
from pathlib import Path

import yaml

CLUSTER_NAME = os.getenv("CLUSTER_NAME", "cluster")
REMOTE_TEMPLATES_PATH = "https://storage.googleapis.com/indico-templates/"


def represent_ordereddict(dumper, data):
    value = []

    for item_key, item_value in data.items():
        node_key = dumper.represent_data(item_key)
        node_value = dumper.represent_data(item_value)

        value.append((node_key, node_value))

    return yaml.nodes.MappingNode("tag:yaml.org,2002:map", value)


yaml.add_representer(OrderedDict, represent_ordereddict)


def merge_dicts(dict1, dict2):
    # dict2 overwrites dict1
    return dict(_merge_dicts(dict1, dict2))


def _merge_dicts(dict1, dict2):
    for k in set(dict1.keys()).union(dict2.keys()):
        if k in dict1 and k in dict2:
            if isinstance(dict1[k], dict) and isinstance(dict2[k], dict):
                yield (k, dict(_merge_dicts(dict1[k], dict2[k])))
            elif (
                k == "env" and isinstance(dict1[k], list) and isinstance(dict2[k], list)
            ):
                resulting_envs = {}
                order = []
                for envvar in dict1[k] + dict2[k]:
                    name = envvar.pop("name")
                    if name not in order:
                        order.append(name)
                    resulting_envs[name] = envvar

                if "PODNAME" in order:
                    # We want to prioritize PODNAME because we tend to use it in other vars
                    order.remove("PODNAME")
                    order.append("PODNAME")

                yield (
                    k,
                    [dict(name=name, **resulting_envs[name]) for name in order[::-1]],
                )
            else:
                # If one of the values is not a dict, you can't continue merging it.
                # Value from second dict overrides one in first and we move on.
                yield (k, dict2[k])
        elif k in dict1:
            yield (k, dict1[k])
        else:
            yield (k, dict2[k])


class ConfigsHolder(dict):
    def __init__(self, config):
        self.config_path = config if isinstance(config, Path) else Path(config)
        self.config_path = self.config_path.resolve()
        assert self.config_path.is_file(), f"Cannot find {self.config_path}."
        self.reload()

    def reset(self, key):
        path = self
        path_keys = key.split(".")
        val_key = path_keys.pop()
        for k in path_keys:
            path = path[k]
        if isinstance(path[val_key], dict):
            for k in path[val_key].keys():
                if path[val_key][k] is not None:
                    self.reset(f"{key}.{k}")
        elif isinstance(path[val_key], list):
            path[val_key] = []
        elif isinstance(path[val_key], int):
            path[val_key] = 0
        else:
            path[val_key] = ""

    def reload(self):
        with open(self.config_path, "r") as f:
            _configs = yaml.safe_load(f)
            self.ordered_keys = list(_configs.keys())
        self.update(merge_dicts(self, _configs))

    def save(self):
        with open(self.config_path, "w") as f:
            resulting_dict = OrderedDict()
            for key in self.ordered_keys:
                resulting_dict[key] = self[key]
            for key, value in self.items():
                if key not in self.ordered_keys:
                    resulting_dict[key] = value
            yaml.dump(resulting_dict, f, default_flow_style=False)


class ConfigsLoader(dict):
    def __init__(self, *configs, **override):
        resolved = {}
        for config in configs:
            resolved = merge_dicts(resolved, self.load(config))

        resolved = merge_dicts(resolved, override)
        self.update(resolved)

    def load(self, config):
        with open(config, "r") as f:
            _configs = yaml.safe_load(f)
        return _configs

    def save(self, output):
        os.makedirs(os.path.dirname(output), exist_ok=True)
        with open(output, "w") as f:
            yaml.dump(dict(self.items()), f, default_flow_style=False)



from ipaddress import ip_network

from click import secho
from indico_install.infra.eks.config import session
from indico_install.infra.eks.infra_utils import gw_info, is_covered_by


@session
def validate(info, session=None):
    """
    Validates the following:
    - defined EKS cluster lies within this VPC
    - defined RDS allows connections within this VPC
    - subnets have necessary cluster tags for K8S
    """
    cluster = session.EKSClient.describe_cluster(name=info["cluster"]["name"])[
        "cluster"
    ]
    vpc = _val_vpc(cluster, session=session)
    subnet_ids = _val_subnets(cluster, vpc, session=session)
    _val_postgres(info["postgres"]["app"]["host"], vpc, subnet_ids, session=session)
    _val_fileshares(info, vpc, subnet_ids, session=session)


@session
def _val_postgres(db_endpoint, vpc, subnet_ids, session=None):
    # Make sure RDS instance is in VPC as well
    # and accessible within certain subnets
    db_name = db_endpoint.split(".")[0]
    postgres = session.RDSClient.describe_db_instances(
        Filters=[
            {
                "Name": "db-cluster-id"
                if "cluster" in db_endpoint
                else "db-instance-id",
                "Values": [db_name],
            }
        ]
    )["DBInstances"]
    assert postgres, f"Unable to find db with name {db_name}"
    postgres = postgres[0]

    assert postgres["DBSubnetGroup"]["VpcId"] == vpc.vpc_id, "DB not in expected VPC"

    # Make sure db is accessible from all cluster subnets
    uncovered_subnet_cidrs = {
        s.subnet_id: ip_network(s.cidr_block)
        for s in vpc.subnets.filter(SubnetIds=subnet_ids)
    }
    db_sgs = ",".join(
        [f'`{sg["DBSecurityGroupName"]}`' for sg in postgres["DBSecurityGroups"]]
    )
    db_vpc_sgs = [sg["VpcSecurityGroupId"] for sg in postgres["VpcSecurityGroups"]]

    if db_sgs:
        _val_postgres_db_sgs(vpc, uncovered_subnet_cidrs, db_sgs, session)

    if db_vpc_sgs:
        _val_postgres_db_vpc_sgs(vpc, uncovered_subnet_cidrs, db_vpc_sgs, session)

    if uncovered_subnet_cidrs:
        secho(
            f"subnets {list(uncovered_subnet_cidrs.keys())} may not covered in the "
            "DB security group rules for your database. Please review (may be a false positive)",
            fg="yellow",
        )


def _val_postgres_db_vpc_sgs(vpc, subnet_cidrs, db_vpc_sgs, session):
    sgs = session.EC2Client.describe_security_groups(GroupIds=db_vpc_sgs)[
        "SecurityGroups"
    ]
    for sub, cidr in dict(subnet_cidrs).items():
        for sg in sgs:
            if sg["VpcId"] == vpc.vpc_id and any(
                [
                    is_covered_by(cidr, ip_network(ipr["CidrIp"]))
                    for ipperm in sg["IpPermissions"]
                    for ipr in ipperm["IpRanges"]
                ]
            ):
                subnet_cidrs.pop(sub)
                break


def _val_postgres_db_sgs(vpc, subnet_cidrs, db_sgs, session):
    all_db_sgs = list(
        session.RDSClient.get_paginator("describe_db_security_groups")
        .paginate(PaginationConfig={"PageSize": 100})
        .search(f"DBSecurityGroups[?contains([{db_sgs}], DBSecurityGroupName)]")
    )
    for sub, cidr in dict(subnet_cidrs).items():
        for sg in all_db_sgs:
            if sg["VpcId"] == vpc.vpc_id and any(
                [
                    is_covered_by(cidr, ip_network(sg_cidr["CIDRIP"]))
                    for ipr in sg["IPRanges"]
                    for sg_cidr in ipr
                ]
            ):
                subnet_cidrs.pop(sub)
                break


@session
def _val_fileshares(info, vpc, subnet_ids, session=None):
    subnet_cidrs = [
        ip_network(s.cidr_block) for s in vpc.subnets.filter(SubnetIds=subnet_ids)
    ]
    gw_data = gw_info(info, session=session)
    shares = [gw_data["indico_fs"], gw_data["client_fs"]]

    for fs in shares:
        clients = [ip_network(client_cidr) for client_cidr in fs["ClientList"]]
        # Make sure all the subnets are covered in the client list of the fileshare
        for cidr in subnet_cidrs:
            if not any([is_covered_by(cidr, client) for client in clients]):
                secho(
                    f"{fs['FileShareId']} may not allow access to subnet CIDR {cidr}",
                    fg="yellow",
                )


@session
def _val_subnets(cluster, vpc, session):
    # Check that subnets are tagged correctly
    subnet_ids = cluster["resourcesVpcConfig"]["subnetIds"]
    private_sub_tag = {"Key": "kubernetes.io/role/internal-elb", "Value": "1"}
    public_sub_tag = {"Key": "kubernetes.io/role/elb", "Value": "1"}
    cluster_tag = {"Key": f"kubernetes.io/cluster/{cluster['name']}", "Value": "shared"}
    cluster_owned_tag = {
        "Key": f"kubernetes.io/cluster/{cluster['name']}",
        "Value": "owned",
    }

    total_ips = 0
    for subnet in vpc.subnets.filter(SubnetIds=subnet_ids):
        total_ips += subnet.available_ip_address_count
        assert (
            subnet.state == "available"
        ), f"Subnet {subnet.subnet_id} is not available"
        assert (
            private_sub_tag in subnet.tags or public_sub_tag in subnet.tags
        ), f"No elb tag on {subnet.subnet_id}"
        assert (
            cluster_tag in subnet.tags or cluster_owned_tag in subnet.tags
        ), f"Missing {cluster_tag} on {subnet.subnet_id}"
    assert total_ips >= 250, "A minimum of 250 IP addresses must be available in total"
    return subnet_ids


@session
def _val_vpc(cluster, session=None):
    # Check that VPC supports DNS and is available
    vpc_id = cluster["resourcesVpcConfig"]["vpcId"]
    assert vpc_id, "Missing VPC id!"
    my_vpc = session.EC2Resource.Vpc(vpc_id)
    assert my_vpc.state == "available"

    errors = 0
    try:
        assert my_vpc.describe_attribute(Attribute="enableDnsSupport")[
            "EnableDnsSupport"
        ]["Value"]
    except Exception:
        secho("Unable to validate that vpc has dns support enabled")
        errors += 1
    try:
        assert my_vpc.describe_attribute(Attribute="enableDnsHostnames")[
            "EnableDnsHostnames"
        ]["Value"]
    except Exception:
        secho("Unable to validate that vpc has dns hostname support enabled")
        errors += 1
    if errors:
        raise Exception("Errors in validating vpc")
    return my_vpc



import click

from indico_install.utils import options_wrapper
from indico_install.infra.input_utils import download_indicoapi_data, auth_with_gsutil
from indico_install.infra.single_node import single_node
from indico_install.infra.gke import gke
from indico_install.infra.aks import aks

# Here we try and include the EKS function
# It depends on boto3 which may not be installed
# in which case we include a stub instead
try:
    from indico_install.infra.eks import eks
except Exception:

    @click.command("eks")
    def eks():
        """Not available"""
        pass


@click.group("infra")
@click.pass_context
def infra(ctx):
    """
    Indico infrastructure setup and validation. Supports EKS, GKE, and single node installations.

    EKS Note! must install package with "eks" extras for EKS
    """
    pass


@infra.command("download-apidata")
@click.pass_context
@click.argument("version", required=True, type=str)
@click.option(
    "--extract/--no-extract",
    default=True,
    show_default=True,
    help="Automatically extract the downloaded TAR",
)
@options_wrapper()
def download_api_data(ctx, version, *, extract, deployment_root, **kwargs):
    """
    Download VERSION of API data TAR from google cloud to local --deployment-root.
    Un-tar the file into a directory of the same name, also in the deployment-root
    VERSION is something like "v7".

    Requires Authentication with GSutil to download the TAR, but will attempt to auth with an existing key if it exists.

    Will not download TAR if it already exists.
    Will not extract the TAR if the data directory already exists in --deployment-root
    """
    auth_with_gsutil(deployment_root)
    download_indicoapi_data(deployment_root, version=version, extract=extract)


for command_group in [single_node, gke, eks, aks]:
    infra.add_command(command_group)



from indico_install.infra.eks.config import session
import json
from indico_install.utils import run_cmd


@session
def validate(config, session=None):
    """
    Validates the following:
    Cluster is in healthy status
    Nodes are up and connected
    """

    # Check the cluster via kubectl
    nodes = [n.split() for n in run_cmd("kubectl get nodes --no-headers").splitlines()]
    assert len(nodes) >= 2, "Minimum 2 nodes required"
    assert all([n[1] == 'Ready' for n in nodes]), "Not all nodes are ready"
    [_validate_node(n[0]) for n in nodes]

    # Now we check to make sure GPU is enabled within the cluster
    nvidia_pods = run_cmd(
        "kubectl get pods --all-namespaces | grep 'nvidia' | grep 'Running'", silent=True
    ).strip()
    assert nvidia_pods != "", "The cluster does not have NVIDIA GPU pods running."


def _validate_node(node_name):
    node_attrs = json.loads(run_cmd(f"kubectl get node {node_name} -o json", silent=True))

    # Test memory
    mem = node_attrs["status"]["capacity"]["memory"]
    mem_units = mem[-2:]
    mem = int(mem[:-2])
    if mem_units.startswith('K'):
        mem = mem / 6
    elif mem_units == 'B':
        mem = mem / 9
    assert mem >= 100.0, "Not enough memory on pods"

    # Check gpu availability on the nodes
    mem = node_attrs["status"]["capacity"].get("nvidia.com/gpu")
    assert mem and int(mem) >= 1, f"GPUs not available on node {node_name}"



import tarfile
from pathlib import Path

from click import prompt, secho

from indico_install.utils import convertb64, run_cmd, find_gcs_key


def auth_with_gsutil(deployment_root, raise_missing_key=False):
    gcs_key = find_gcs_key(deployment_root, quiet=not raise_missing_key)
    if not gcs_key:
        if raise_missing_key:
            raise AssertionError("Unable to find a key for Google auth")
        else:
            return

    authed = run_cmd(f"gcloud auth activate-service-account --key-file={gcs_key} 2>&1", silent=True)
    return gcs_key if "Activated service account" in authed else None


def postgres_input(conf):
    db_details = conf["postgres"]["app"]
    host = prompt("What is your DB endpoint?", type=str, default=db_details.get("host"))
    user = prompt("What is your DB user?", type=str, default=db_details.get("user"))
    password = prompt(
        "What is your DB password? Type 'USE OLD' to use existing pw.",
        hide_input=True,
        confirmation_prompt=True,
        default="USE OLD" if db_details.get("password") else None,
    )
    password = (
        db_details.get("password") if password == "USE OLD" else convertb64(password)
    )
    conf["postgres"]["app"].update({"host": host, "password": password, "user": user})


def download_indicoapi_data(deployment_root, version=None, extract=True):
    indicoapi_version = version or prompt("Version of api model data to download", default="v7")
    indicoapi_tar = f"indicoapi_data_{indicoapi_version}.tar.gz"

    tar_path = Path(deployment_root) / indicoapi_tar
    data_path = Path(deployment_root) / f"indicoapi_data_{indicoapi_version}"
    if not data_path.exists():
        if tar_path.exists():
            secho(f"Skipping download - using {tar_path}", fg="yellow")
        else:
            run_cmd(
                f"mkdir -p {data_path} && gsutil cp gs://indicoapi-data/{indicoapi_tar} {deployment_root}"
            )
            assert tar_path.exists(), "Unable to download indicoapi_data successfully"
        if extract:
            tar = tarfile.open(tar_path, "r:gz")
            tar.extractall(path=data_path)
            tar.close()
            secho(f"Unzipped data to {data_path}", fg="green")
        else:
            secho("Please extract and upload TAR {tar_path} when ready")
            return
    else:
        secho(
            f"Skipping indicoapi data download - already exists at {data_path}",
            fg="yellow",
        )

    return data_path



import tempfile
import time
import uuid
from pathlib import Path

import click
import yaml

from indico_install.config import ConfigsHolder, merge_dicts
from indico_install.helm.render import _resolve_remote, render
from indico_install.kube.svc.restart import restart
from indico_install.setup import configure_gcr_key
from indico_install.utils import current_user, options_wrapper, run_cmd


def diff_files(f1, f2):
    diff_out = run_cmd(f"diff {f1} {f2}", silent=True)
    for line in diff_out.splitlines():
        if line.startswith("+"):
            click.secho(line, fg="green")
        elif line.startswith("-"):
            click.secho(line, fg="red")
        else:
            click.echo(line)


class Deployment:
    def __init__(self):
        self.key = str(uuid.uuid4())
        self.start = time.time()
        self.user = current_user()

    def get_deployment_version_data(self):
        curr_vers = run_cmd(
            "kubectl get configmap deployed-version -o yaml 2>&1", silent=True
        )
        assert (
            "NotFound" not in curr_vers
        ), "Configmap for current version not found! Please indico apply to initialize the cluster"
        return yaml.safe_load(curr_vers)["data"]

    def set_deployment_version(self, services_yaml_version, templates_version=None):
        run_cmd(
            """kubectl patch configmap deployed-version -p '{"data":{"services.yaml": "%s","templates": "%s","updated_by":"%s", "last_updated":"%s"}}'"""
            % (
                services_yaml_version,
                templates_version or services_yaml_version,
                self.user,
                time.strftime("%m/%d/%y %H:%M:%S %z", time.localtime(self.start)),
            )
        )

    def is_version_locked(self):
        # lock exists, wasn't set by us, and is less than 10 min old
        curr_lock = self.get_deployment_version_data().get("lock", {})
        return (
            curr_lock
            and curr_lock["applied_by"] != self.key
            and int(self.start - int(curr_lock["time"])) < 10 * 60
        )

    def set_deployment_lock(self, remove=False, wait=0):
        while wait:
            if self.is_version_locked():
                time.sleep(1)
                wait -= 1
            else:
                break
        assert not self.is_version_locked(), "deploymet is already locked"
        lock_val = (
            "null"
            if remove
            else '{"applied_by":"%s","time":"%d"}' % (self.key, self.start)
        )
        run_cmd(
            """kubectl patch configmap deployed-version -p '{"data":{"lock":%s}}'"""
            % lock_val,
            silent=True,
        )


@click.command("info")
@click.pass_context
def info(ctx):
    """
    Grab the deployed-version configmap
    for information on the state of the cluster
    """
    click.echo(Deployment().get_deployment_version_data())


@click.command("apply-patch")
@click.pass_context
@click.option("-f", "--patch-file", required=True, help="Patch file to apply")
@click.option(
    "-w",
    "--wait",
    type=int,
    default=0,
    show_default=True,
    help="Seconds to wait for deploymewnt to unlock",
)
@options_wrapper(check_input=True)
def apply_patch(ctx, patch_file=None, wait=0, *, deployment_root, yes, **kwargs):
    """
    Apply a patch to the current services - this will NOT override params in cluster.yaml
    Example patch yaml to update the moonbow image for all moonbow services

      images: {moonbow: moonbow:development.32532464564574637}
    """
    try:
        from indico_install.helm.upload import upload
    except ImportError:
        click.secho("Unable to apply patch without indico pip dependencies", fg="red")
        return

    if not Path(patch_file).is_file():
        click.secho(f"Could not find patch file {patch_file}", fg="red")
        return
    dep = Deployment()
    curr_vers = dep.get_deployment_version_data()
    # curr_vers = {"services.yaml": "development", "templates": "development"}
    dep.set_deployment_lock(wait=wait)

    try:
        with open(patch_file, "r") as f:
            patches = yaml.safe_load(f)

        services_yaml, templates_dir = _resolve_remote(
            Path(deployment_root), curr_vers["services.yaml"]
        )
        new_services_file = Path(tempfile.gettempdir()) / "services.yaml"
        run_cmd(f"cp {services_yaml} {new_services_file}")
        new_services = ConfigsHolder(new_services_file)
        new_services.update(merge_dicts(new_services, patches))
        new_services.save()
        diff_files(services_yaml, new_services_file)

        if yes or click.confirm("Apply changes?"):
            run_cmd(f"mv {new_services_file} {services_yaml}", silent=True)

            new_version = f"{dep.start}_{dep.user}"

            versions = ctx.invoke(
                upload,
                tag=[new_version],
                deployment_root=templates_dir.parent,
                services_yaml=services_yaml,
            )
            new_version = versions[0]
            ctx.invoke(
                render, deployment_root=deployment_root, remote_configs=new_version
            )
            ctx.invoke(apply, deployment_root=deployment_root, yes=True, **kwargs)
            dep.set_deployment_version(new_version)
    finally:
        dep.set_deployment_lock(remove=True)


def find_dependent_services(updated_services: list) -> set:
    """
    From a list of services, determine if any configs or secrets have been updated
    and return a list of deployment and statefulset names that need to be restarted
    to pick up changes. Determined if the services use any of the <updated_confs>
    in their envFrom or volumes.
    """
    svc_types_to_check = ("statefulset", "deployment", "configmap", "secret")
    updated_confs = [
        ups.split("/", 1)[1]
        for ups in updated_services
        if ups.startswith("secret/") or ups.startswith("configmap/")
    ]
    updated_services = {
        ups.split("/", 1)[1]: idx
        for idx, ups in enumerate(updated_services)
        if any(ups.startswith(svc_type) for svc_type in svc_types_to_check)
    }
    # We bank on the fact that names of deployments, statefulsets, configs, secrets are all unique

    dependency_info = []
    dependent_services = set([])
    # 1. Get all the conf dependencies for our resources
    for svc_type in ("deployment", "statefulset"):
        dependency_info.extend(
            run_cmd(
                f"kubectl get {svc_type} "
                "--no-headers -o custom-columns=NAME:.metadata.name,"
                "ENVFROM:{.spec.template.spec.containers[0].envFrom[*]},"
                "CONFIGMAPVOL:{.spec.template.spec.volumes[*]}",
                silent=True,
            ).splitlines()
        )

    # 2. For each conf, find the list of dependent resources
    for conf in updated_confs:
        conf_order = updated_services[conf]
        conf_deps = set([ds.split("  ", 1)[0] for ds in dependency_info if conf in ds])
        # remove any dependencies that were updated after the conf was updated
        conf_deps -= set(
            [cd for cd in conf_deps if updated_services.get(cd, 0) > conf_order]
        )
        dependent_services.update(conf_deps)

    return dependent_services


@click.command("apply")
@click.pass_context
@click.argument("service", required=False)
@click.option(
    "--restart-deps/--no-restart-deps",
    default=True,
    show_default=True,
    help="Restart services affected by updated configmap or secrets",
)
@options_wrapper()
def apply(ctx, restart_deps=True, service=None, deployment_root=None, yes=False, **kwargs):
    """
    Apply templates in the "generated/" directory.
    Apply only the template files matching <SERVICE> if provided.
    Creates the image pull secret if missing.
    Performs a dry-run for confirmation before applying. Skip dry-run with --yes
    """
    generated_dir = Path(deployment_root) / "generated"
    if not generated_dir.is_dir():
        click.secho(f"'generated/' directory not found in {deployment_root}")
        return

    configure_gcr_key(deployment_root)

    # These are services that were actually modified
    updated_services = []

    if service is not None:
        templates = run_cmd(
            f"ls {generated_dir} | grep '{service}'", silent=True
        ).splitlines()
        for t in templates:
            results = run_cmd(
                f"kubectl apply -f {generated_dir / t} --dry-run 2>&1 | grep -v 'support dry-run'"
            )

        if yes or click.confirm("Ready to apply these changes?"):
            outs = [run_cmd(f"kubectl apply -f {generated_dir / t}") for t in templates]
            updated_services.extend(
                [
                    out.split(" ", 1)[0]
                    for out in outs
                    if any(status in out for status in ("created", "configured"))
                ]
            )
    else:
        results = run_cmd(
            f"kubectl apply -R -f {generated_dir} --dry-run 2>&1 | grep -v 'support dry-run'",
            silent=True,
        )
        click.echo("\n".join(results.split(",")) + "\n")

        if yes or click.confirm("Ready to apply these changes?"):
            out = run_cmd(f"kubectl apply -R -f {generated_dir}", silent=True)
            for line in out.splitlines():
                color = "red"
                if "unchanged" in line:
                    color = "yellow"
                elif "configured" in line or "created" in line:
                    color = "green"
                    updated_services.append(line.split(" ", 1)[0])
                click.secho(line, fg=color)

    # Now we reroll any dependent services as necessary
    if updated_services:
        updated_confs = [
            ups
            for ups in updated_services
            if ups.startswith("secret/") or ups.startswith("configmap/")
        ]
        if not updated_confs:
            return
        if restart_deps:
            dependent_services = find_dependent_services(updated_services)
            if not dependent_services:
                click.secho(
                    f"No dependent services found that need to be restarted",
                    fg="yellow",
                )
                return
            for dep_service in dependent_services:
                ctx.invoke(restart, service=dep_service, contains=False, wait="2m")
        else:
            click.secho(f"{', '.join(updated_confs)} were updated. Make sure to restart dependent services")



import os
import uuid
from click import secho, prompt, confirm

from indico_install.config import ConfigsHolder
from indico_install.utils import base64file, find_values, convertb64, run_cmd, find_gcs_key


def configure_passwords(configs):
    """Generate secrets for <generate_secret> inputs"""
    for _dict, key, _ in find_values(configs, "<generate_secret>"):
        _dict[key] = convertb64(uuid.uuid4().hex[:32])

    configs.save()

def configure_gcr_key(deployment_root):
    if run_cmd("kubectl get secrets | grep gcr-pull-secret", silent=True):
        return

    key_file = find_gcs_key(deployment_root)
    if not key_file:
        return

    secho("Generating gcr-pull-secret", fg="yellow")
    client_name = key_file[4:-5]
    out = run_cmd(
        (
            'kubectl create secret docker-registry gcr-pull-secret '
            '--docker-server=https://gcr.io --docker-username=_json_key '
            f'--docker-password="$(cat {deployment_root}/gcr-{client_name}.json)" '
            f'--docker-email="gcr-{client_name}@new-indico.iam.gserviceaccount.com"'
        ), silent=True
    )
    secho(f"{out}\n", fg="green")


def get_tls(configs, redo=False):
    if not redo and all([v for v in configs["tls"].values()]):
        return
    cert_file = configs["tls"].get("certfile")
    key_file = configs["tls"].get("keyfile")

    secho("Ctrl-C will skip the prompt.", fg="blue")

    try:
        ok = False
        while not ok:
            cert_file = os.path.abspath(prompt("SSL Cert File", default=cert_file if cert_file else None))
            ok = confirm(f"Is this the file? {cert_file}")
        configs["tls"]["cert"] = base64file(cert_file)
        configs["tls"]["certfile"] = cert_file
    except Exception:
        secho("Invalid ssl cert file", fg="red")
        return

    try:
        ok = False
        while not ok:
            key_file = os.path.abspath(prompt("SSL Key File", default=key_file if key_file else None))
            ok = confirm(f"Is this the file? {key_file}")
        configs["tls"]["key"] = base64file(key_file)
        configs["tls"]["keyfile"] = key_file
    except Exception:
        secho("Invalid ssl key file", fg="red")
        return

    configs.save()


def setup(input_yaml):
    """
    Set up your initial environment using an interactive yaml form
    """

    configs = ConfigsHolder(config=input_yaml)
    configure_passwords(configs)

    for k in ["parentDomain", "publicDomain"]:
        if not configs.get(k):
            configs[k] = prompt(k)
    configs.save()

    get_tls(configs)



import click
from indico_install.helm.apply import apply, apply_patch, info
from indico_install.helm.render import render

# Here we try and include the upload function
# It depends on google-cloud-storage which may not be installed
# in which case we include a stub instead
try:
    from indico_install.helm.upload import upload
except ImportError:
    @click.command("upload")
    def upload():
        """Not available"""
        pass



import click
from indico_install.utils import run_cmd


@click.command("restart")
@click.pass_context
@click.argument("service", required=True)
@click.option("--wait", default="0", show_default=True, help="Amount of time to wait for the new pods to be ready. Ex: 3m, 4h")
@click.option(
    "--contains/--no-contains",
    default=True,
    show_default=True,
    help="Restart all services with names containing the <service> string. Only applicable if <service> is not 'all'",
)
@click.option(
    "--all",
    "all_services",
    is_flag=True,
    default=False,
    help="Update only indico services (default) or all services in the current namespace (--all). Only applicable for service=all",
)
def restart(ctx, service, all_services=False, contains=True, wait="0"):
    """
    Reroll a K8S cluster deployment or statefulset.

    ARGS:

        <SERVICE> grep string of deployments and statefulsets to reroll

         - "all" reset all deployments and statefulsets with an inditype label of service or celerytask
    """
    for svc_type in ["deployment", "statefulset"]:
        wait_str = "--watch=false" if wait == "0" else f"--timeout={wait}"
        if service == "all":
            label_match = (
                "" if all_services else "-l 'inditype in (service, celerytask)'"
            )
            services = run_cmd(
                f"""kubectl get {svc_type} {label_match} --no-headers -o custom-columns=NAME:.metadata.name""",
                silent=True).splitlines()
            for svc_to_update in services:
                svc_to_update = f"{svc_type}/{svc_to_update}"
                run_cmd(f"kubectl rollout restart {svc_to_update} && kubectl rollout status {svc_to_update} {wait_str}")
        else:
            service_str = service if contains else f"^{service}"
            services = run_cmd(
                f"kubectl get {svc_type} --no-headers -o custom-columns=NAME:.metadata.name | grep '{service_str}'",
                silent=True).splitlines()
            for svc_to_update in services:
                svc_to_update = f"{svc_type}/{svc_to_update}"
                run_cmd(f"kubectl rollout restart {svc_to_update} && kubectl rollout status {svc_to_update} {wait_str}")



import base64
import os
import pty
import subprocess
from pathlib import Path
from functools import wraps
import click

from indico_install.config import CLUSTER_NAME


def options_wrapper(check_input=False, check_services=False):
    def opt_wrapper_outer(f):
        @wraps(f)
        @click.option(
            "-d",
            "--deployment-root",
            default=os.getcwd(),
            show_default=True,
            help="Root directory for installation files",
        )
        @click.option(
            "-c",
            "--cluster",
            default=CLUSTER_NAME,
            envvar="CLUSTER_NAME",
            show_default=True,
            help="Name of your cluster - can specify the input file to use",
        )
        @click.option(
            "-i",
            "--input-yaml",
            help="Input filepath - defaults to --deployment-root/values/--cluster.yaml",
        )
        @click.option(
            "-s",
            "--services-yaml",
            help="Services filepath - defaults to --deployment-root/values/services.yaml",
        )
        @click.option(
            "--yes/--no",
            "-y/-n",
            default=False,
            show_default=True,
            help="Accept changes",
        )
        def wrapped(
            *args,
            deployment_root=False,
            cluster=None,
            input_yaml=None,
            services_yaml=False,
            yes=False,
            **kwargs,
        ):
            # Backend
            input_yaml = (
                Path(input_yaml)
                if input_yaml
                else Path(deployment_root) / "values" / f"{cluster}.yaml"
            )
            services_yaml = (
                Path(services_yaml)
                if services_yaml
                else Path(deployment_root) / "values" / "services.yaml"
            )
            if check_input and not input_yaml.is_file():
                click.secho(f"Could not find {input_yaml}.", fg="red")
                return
            if check_services and not services_yaml.is_file():
                click.secho(f"Could not find {services_yaml}.", fg="red")
                return

            return f(
                *args,
                deployment_root=deployment_root,
                cluster=cluster,
                input_yaml=input_yaml,
                services_yaml=services_yaml,
                yes=yes,
                **kwargs,
            )

        return wrapped
    return opt_wrapper_outer


def _read(fd):
    return os.read(fd, 10240)


def with_pty(cmd: list):
    pty.spawn(cmd, master_read=_read, stdin_read=_read)


def run_cmd(mycmd, envvars: dict = None, silent=False, tty=False):
    if envvars is None:
        envvars = dict(os.environ)
    if tty and not isinstance(mycmd, list):
        raise AssertionError("Command must be passed in as a list for TTY")
    if not silent:
        click.secho(f"Running command: {mycmd}", fg="blue")

    popen_kwargs = dict(env=envvars, shell=True, stdout=subprocess.PIPE)
    if tty:
        return with_pty(mycmd)
    else:
        output = subprocess.Popen(mycmd, **popen_kwargs)
        stdout, _ = output.communicate()
        if stdout is not None:
            stdout = stdout.decode("utf-8").strip()
            if not silent:
                print(stdout, "\n")
        return stdout


def region_to_gmt(region):
    offset = "+0:00"
    if region.upper().startswith("us-east"):
        offset = "-5:00"
    return f"GMT{offset}"


def get_value(value_dict, keys):
    if not keys:
        return value_dict
    elif len(keys) == 1:
        return value_dict.get(keys[0], None)
    else:
        return get_value(value_dict[keys[0]], keys[1:])


def find_values(value_dict, target_value, base_key=None):
    if base_key is None:
        base_key = []
    for key, value in value_dict.items():
        if isinstance(value, dict):
            yield from find_values(value, target_value, base_key=base_key + [key])
        elif isinstance(value, str) and value.strip().lower() == target_value:
            yield value_dict, key, base_key


def base64file(filename):
    with open(filename, "r") as f:
        return convertb64(f.read())


def convertb64(mystring):
    return base64.b64encode(mystring.encode("utf-8")).decode("utf-8")

def decodeb64(mystring):
    return base64.b64decode(mystring.encode("utf-8")).decode("utf-8")


def determine_resource(deployment):
    for resource in ("deployment", "statefulset"):
        result = run_cmd(
            f"kubectl get {resource} | tail -n +2 | awk '{{print $1}}'", silent=True
        )
        if deployment in result.split():
            return resource


def current_user(clean=False):
    try:
        user = next(
            (
                s
                for s in run_cmd("gcloud auth list 2>/dev/null", silent=True).splitlines()
                if s.startswith("*")
            ),
            "no auth",
        ).split()[1]
        if clean:
            user = user.split('@', 1)[0].replace('gcr-', '')
        return user
    except Exception:
        return None


def find_gcs_key(deployment_root, quiet=False):
    # TODO: improve the gcr key finder
    key_file = run_cmd(f"ls {deployment_root}" + r" | grep -e '^gcr-.*\.json$'", silent=True)
    err = None
    if not key_file:
        err = "No GCR key file found in {deployment_root}! A GCR key file looks like 'gcr-<name>.json'"
    elif len(key_file.split("/n")) > 1:
        err = "Too many GCR key files found. Please only have 1"

    if err:
        if not quiet:
            click.secho(err, fg="red")
        return None
    return key_file


def string_to_tag(string):
    """Clean string to be used as folder path"""
    return "".join(
        [c if c.isalnum() or c in ("-", "_") else "_" for c in (string or "")]
    )
